package dbs.ws15;

import dbs.DBConnector;

import java.sql.*;
import java.util.HashMap;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.io.IOException;

public class Szenario1 {

    private Connection connection = null;

    public static void main(String[] args) {
        if (args.length <= 6 && args.length >= 4) {
            /*
             * args[0] ... type -> [a|b], 
             * args[1] ... server, 
             * args[2] ... port,
             * args[3] ... database, 
             * args[4] ... username, 
             * args[5] ... password
             */

            Connection conn = null;

            if (args.length == 4) {
                conn = DBConnector.getConnection(args[1], args[2], args[3]);
            } 
            else {
                if (args.length == 5) {
                    conn = DBConnector.getConnection(args[1], args[2], args[3], args[4], "");
                } 
                else {
                    conn = DBConnector.getConnection(args[1], args[2], args[3], args[4], args[5]);
                }
            }

            if (conn != null) {
                Szenario1 s = new Szenario1(conn);

                if (args[0].equals("a")) {
                    s.runTransactionA();
                } 
                else {
                    s.runTransactionB();
                }
                
                try {
					conn.close();
				} catch (SQLException ex) {
					Logger.getLogger(Szenario1.class.getName()).log(Level.SEVERE, null, ex);
				}
            }
        } 
        else {
            System.err.println("Ungueltige Anzahl an Argumenten!");
        }
    }

    public Szenario1(Connection connection) {
        this.connection = connection;
    }

    /*
     * Beschreibung siehe Angabe
     */
    public void runTransactionA() {
        /*
         * Vorgegebener Codeteil
         * ################################################################################
         */
    	
    	
    	
        wait("Druecken Sie <ENTER> zum Starten der Transaktion ...");
        /*
         * ################################################################################
         */
        
        System.out.println("Transaktion A Start");

        
        /*
         * Setzen Sie das aus Ihrer Sicht passende Isolation-Level:
         */
        
        try {
        
            connection.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
            

        /*
         * Abfrage 1:
         * Anzahl der Auftraege pro Kunden
         */
            
            Statement stmt = connection.createStatement();
            
            ResultSet rs1 = stmt.executeQuery("SELECT k.knr, k.name, count(anr)  AS anz " +
                                              "FROM Kunde k JOIN Auftrag a ON k.knr = a.knr " + 
                                              "GROUP BY k.knr, k.name;");
       /*
        * Ausgeben der Anzahl der Auftraege pro Kunden
        */
            System.out.println("Anzahl der Auftraege pro Kunden:");
            System.out.println("-------------------------------------");            
            while (rs1.next()) {
            	System.out.println("Kunde " + rs1.getString(2) +": " + rs1.getInt(3));
            }
            System.out.println();
            rs1.close();
            
        /*
         * Vorgegebener Codeteil
         * ################################################################################
         */
        wait("Druecken Sie <ENTER> zum Fortfahren ...");
        /*
         * ################################################################################
         */
        
        /*
         * Abfrage 2:
         * Anzahl der Auftraege
         */
        
        ResultSet rs2 = stmt.executeQuery("SELECT count(anr) FROM auftrag;");
        rs2.next();
        
        System.out.println("Anzahl der Auftraege: " + rs2.getInt(1));
        rs2.close();

        /*
         * Vorgegebener Codeteil
         * ################################################################################
         */
        wait("Druecken Sie <ENTER> zum Beenden der Transaktion ...");
        /*
         * ################################################################################
         */
        
        stmt.close();
        
        } 
        catch (SQLException ex) {
            Logger.getLogger(Szenario1.class.getName()).log(Level.SEVERE, null, ex);
        }

        /*
         * Beenden Sie die Transaktion
         */

        System.out.println("Transaktion A Ende");
    }

    public void runTransactionB() {
        /*
         * Vorgegebener Codeteil
         * ################################################################################
         */
        wait("Druecken Sie <ENTER> zum Starten der Transaktion ...");

        System.out.println("Transaktion B Start");
        
        try {
            Statement stmt = connection.createStatement();
            
            
            stmt.executeUpdate("INSERT INTO Auftrag VALUES (50,1,'2015-11-05');");

            stmt.close();
            
            System.out.println("Ein Auftrag wurde hinzugefuegt ...");
            
            wait("Druecken Sie <ENTER> zum Beenden der Transaktion ...");

            connection.commit();
            
            wait("Druecken Sie <ENTER> zum Beenden des Szenarios ...");
            stmt = connection.createStatement();
            stmt.executeUpdate("DELETE FROM Auftrag WHERE anr = '50'");
            stmt.close();
            connection.commit();
            
        } 
        catch (SQLException ex) {
            Logger.getLogger(Szenario1.class.getName()).log(Level.SEVERE, null, ex);
        }

        System.out.println("Transaktion B Ende");
        /*
         * ################################################################################
         */
    }

    private static void wait(String message) {
        /* 
         * Vorgegebener Codeteil 
         * ################################################################################
         */
         System.out.println(message);
         Scanner scanner = new Scanner(System.in);
         scanner.nextLine();
         /*
         * ################################################################################
         */
    }
}
