package dbs.ws15;

import java.sql.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Date;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeMap;

import dbs.DBConnector;

public class Szenario2 {

    private Connection connection = null;
    private PreparedStatement pstmt_kundendaten = null;
    private PreparedStatement pstmt_kunde = null;

    
    public static void main(String[] args) {
        if (args.length <= 5 && args.length >= 3) {
            /*
             * args[0] ... server, 
             * args[1] ... port,
             * args[2] ... database, 
             * args[3] ... username, 
             * args[4] ... password
             */

            Connection conn = null;

            if (args.length == 3) {
                conn = DBConnector.getConnection(args[0], args[1], args[2]);
            } 
            else {
                if (args.length == 4) {
                    conn = DBConnector.getConnection(args[0], args[1], args[2], args[3], "");
                } 
                else {
                    conn = DBConnector.getConnection(args[0], args[1], args[2], args[3], args[4]);
                }
            }

            if (conn != null) {
                Szenario2 s = new Szenario2(conn);

                s.prepareStatement();
                s.run();
            }

        } 
        else {
            System.err.println("Ungueltige Anzahl an Argumenten!");
        }
    }

	public Szenario2(Connection connection) {
        this.connection = connection;
    }
	
	/*
     * Hier das Statement erstellen
     */
	private void prepareStatement() {
		// TODO Write your code here
		
		try {
			pstmt_kundendaten = connection.prepareStatement("SELECT * FROM AktKundendaten WHERE knr = ?");
			pstmt_kunde = connection.prepareStatement("SELECT name FROM Kunde WHERE knr = ?");
		} catch (SQLException ex) {
			Logger.getLogger(Szenario1.class.getName()).log(Level.SEVERE, null, ex);
		}
	}

    /*
     * Fuer jeden Mitarbeiter printAkteneintrag aufrufen
     */
    public void run() {
    	// TODO Write your code here
        int knr = -1;
        boolean knr_changed=true;
        Map<Integer,Integer> cnt_archiv_knr = new TreeMap<Integer,Integer>(); 
        int count = 0;
    	
    	try {
			Statement stmt = connection.createStatement(ResultSet.CONCUR_UPDATABLE,
					ResultSet.TYPE_SCROLL_INSENSITIVE);
			
			ResultSet rs = stmt.executeQuery("SELECT * FROM Kundendaten ORDER BY knr;");
			
			
			while (rs.next()) {
				if (rs.getInt(1) != knr) knr_changed=true;
				knr = rs.getInt(1);
				Date date = rs.getDate(2);
				
				pstmt_kundendaten.setInt(1, knr);
				
				ResultSet rs_akt = pstmt_kundendaten.executeQuery();
				rs_akt.next();			
				
				if (!date.equals(rs_akt.getDate(2))) {
					printKundendaten(rs);
					rs.deleteRow();
					cnt_archiv_knr.put(knr, cnt_archiv_knr.containsKey(knr)?cnt_archiv_knr.get(knr)+1:1);
				}
				
				rs_akt.close();
			}
			
			rs.close();
			stmt.close();
			
			connection.commit();
			
			System.out.println();
			
			for (Entry<Integer,Integer> entry : cnt_archiv_knr.entrySet()) {
				pstmt_kunde.setInt(1, entry.getKey());
				pstmt_kunde.execute();
				rs = pstmt_kunde.executeQuery();
				rs.next();
				System.out.println("Anzahl der archivierten Eintraege fuer [" + entry.getKey() + "] " + rs.getString(1) + ": " + entry.getValue());
				rs.close();
				count+=entry.getValue();
			}
			
			System.out.println("--------------------------------------------------");
			System.out.println("Gesamtanzahl an archivierten Eintraegen: " + count);
				
		} catch (SQLException ex) {
			Logger.getLogger(Szenario1.class.getName()).log(Level.SEVERE, null, ex);
		}		
    	
    	
    }
    
    /*
     * Kundendaten ausgeben
     */
    private void printKundendaten(ResultSet rs) throws SQLException {
		//TODO Write your code here
    	int knr = rs.getInt(1);
    	Date date = rs.getDate(2);
    	String anschrift = rs.getString(3);
    	int rabatt = rs.getInt(4);
		
		System.out.println("INSERT INTO Kundendaten VALUES (" + knr + ",'"+date + "','"+anschrift + "',"+rabatt+");");
	}

	
    
}
