package dbs.ws15;

import java.sql.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.Date;

import dbs.DBConnector;

public class Szenario3 {

    private Connection connection = null;
    private PreparedStatement pstmt_rpos = null;
    private PreparedStatement pstmt_kunde = null;
    
    public static void main(String[] args) {
        if (args.length <= 5 && args.length >= 3) {
            /*
             * args[1] ... server, 
             * args[2] ... port,
             * args[3] ... database, 
             * args[4] ... username, 
             * args[5] ... password
             */

            Connection conn = null;

            if (args.length == 3) {
                conn = DBConnector.getConnection(args[0], args[1], args[2]);
            } 
            else {
                if (args.length == 4) {
                    conn = DBConnector.getConnection(args[0], args[1], args[2], args[3], "");
                } 
                else {
                    conn = DBConnector.getConnection(args[0], args[1], args[2], args[3], args[4]);
                }
            }

            if (conn != null) {
                Szenario3 s = new Szenario3(conn);

                s.prepareStatement();
                s.run();
            }

        } 
        else {
            System.err.println("Ungueltige Anzahl an Argumenten!");
        }
    }

	public Szenario3(Connection connection) {
        this.connection = connection;
    }
	
	/*
     * Hier das Statement erstellen
     */
	private void prepareStatement() {
		// TODO Write your code here
		
		try {
			pstmt_rpos = connection.prepareStatement("SELECT * FROM RPosition WHERE rnr = ?");	
			pstmt_kunde = connection.prepareStatement("SELECT k.knr, k.name, a.anschrift " +
			                                          "FROM Kunde k JOIN AktKundendaten a ON k.knr = a.knr "
			                                          + "JOIN Auftrag af ON af.knr = k.knr "
			                                          + "JOIN APosition ap ON af.anr = ap.anr "
			                                          + "JOIN RPosition rp ON ap.anr = rp.anr AND ap.apnr = rp.apnr "
			                                          + "WHERE rp.rnr = ?");
		} catch (SQLException ex) {
			Logger.getLogger(Szenario1.class.getName()).log(Level.SEVERE, null, ex);
		}
	}

    /*
     * Fuer jeden Mitarbeiter printAkteneintrag aufrufen
     */
    public void run() {
    	// TODO Write your code here

    	try {
			Statement stmt = connection.createStatement();
			
			ResultSet rs_rechnung = stmt.executeQuery("SELECT * FROM Rechnung;");
			
			while (rs_rechnung.next()) {
				int rnr = rs_rechnung.getInt(1);
				Date datum = rs_rechnung.getDate(2);
				double gesamt = 0.;
				
				pstmt_kunde.setInt(1, rnr);
				ResultSet rs_kunde = pstmt_kunde.executeQuery();
				rs_kunde.next();
				
				System.out.println("-------------------------------------------------------------");
				System.out.println("Rechnung #" + rnr);
				System.out.println("-------------------------------------------------------------");

				System.out.println(rs_kunde.getString(2));
				System.out.println(rs_kunde.getString(3));
				
				pstmt_rpos.setInt(1, rnr);
				ResultSet rs_rpos = pstmt_rpos.executeQuery();
				
				System.out.println();
				System.out.println(" Pos  |                  Text                    |  Betrag");
				System.out.println("------+------------------------------------------+-----------");
				while (rs_rpos.next()) {
					System.out.print(String.format("%5s",rs_rpos.getInt(2)) + " | ");
					System.out.print(String.format("%-40s",rs_rpos.getString("text")) + " | ");
					System.out.println(String.format("%10.2f",rs_rpos.getDouble("wert")));
					gesamt += rs_rpos.getDouble("wert");
				}
				
				System.out.println("-------------------------------------------------+-----------");
				System.out.println("                                        Gesamt : | " + String.format("%10.2f",gesamt));
				System.out.println("-------------------------------------------------------------");
				
				System.out.println();
			}
			
			rs_rechnung.close();
			
				
		} catch (SQLException ex) {
			Logger.getLogger(Szenario1.class.getName()).log(Level.SEVERE, null, ex);
		}		
    }

}
