﻿--Trigger 1--
--Rechnungspositionen einer Rechnung gehören immer zum selben Auftrag--
CREATE OR REPLACE FUNCTION check_rechnung() RETURNS TRIGGER AS $$
BEGIN
	IF EXISTS (SELECT *
				FROM RPosition r 
				WHERE r.rnr = NEW.rnr AND r.anr != NEW.anr) THEN
	    RAISE EXCEPTION 'RPosition hat nicht den selben Auftrag';
		RETURN NULL;
	END IF;
	
	RETURN new;

END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER t_check_rechnung BEFORE INSERT ON RPosition
FOR EACH ROW EXECUTE PROCEDURE check_rechnung();



--Trigger 2--
--Eine Auftragsposition kann entweder eine Sonstige Position oder eine 
--Leistungsposition sein (nicht beides)--
CREATE OR REPLACE FUNCTION check_apos() RETURNS TRIGGER AS $$
DECLARE
BEGIN
	IF EXISTS (SELECT anr, apnr FROM SonstigePosition WHERE anr = NEW.anr AND apnr = NEW.apnr UNION 
	           SELECT anr, apnr FROM Leistungsposition WHERE anr = NEW.anr AND apnr = NEW.apnr) THEN
	RAISE EXCEPTION 'Leistungsposition kann nicht SonstigePosition sein (und umgekehrt)';
	END IF;

	RETURN new;

END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER t_check_spos BEFORE INSERT ON SonstigePosition
FOR EACH ROW EXECUTE PROCEDURE check_apos();

CREATE TRIGGER t_check_lpos BEFORE INSERT ON Leistungsposition
FOR EACH ROW EXECUTE PROCEDURE check_apos();


--Trigger 3--
--Leistungen dürfen nicht zirkulär voneinander abhängig sein. Z.B. kann die Leistung mit 
--der Nummer 1 nicht die Nummer 2 benoetigen welche wiederum die Leistung Nummer 1 
--benoetigt (mit beliebig vielen Zwischenstationen)
CREATE OR REPLACE FUNCTION check_benoetigt() RETURNS TRIGGER AS $$
DECLARE
BEGIN
	IF EXISTS (WITH RECURSIVE temp(lnr,benoetigt) AS (
			     SELECT lnr, benoetigt
			     FROM benoetigt		
		       UNION
			     SELECT t.lnr, b.benoetigt
			     FROM temp t JOIN benoetigt b ON t.benoetigt = b.lnr)
	           SELECT * FROM temp t WHERE t.lnr = NEW.benoetigt AND t.benoetigt = NEW.lnr) THEN 
	RAISE EXCEPTION 'Leistungen haben eine zirkulaere Abhaengigkeit!';
	END IF;

	RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER t_check_benoetigt BEFORE INSERT ON benoetigt
FOR EACH ROW EXECUTE PROCEDURE check_benoetigt();



--Function 1--
--TODO--
--Schreiben Sie eine Funktion, welche als Parameter die Auftragsnummer und die APNR
--einer Auftragsposition erhält
CREATE OR REPLACE FUNCTION f_calc_apos(fanr INTEGER, fapnr INTEGER) RETURNS NUMERIC(7,2) AS $$ 
DECLARE
	vwert NUMERIC(7,2);
	srabatt INTEGER;
	prabatt INTEGER;
	found BOOLEAN;
BEGIN

	found := false;
	
	IF NOT EXISTS(SELECT * FROM APosition a WHERE a.anr = fanr AND a.apnr = fapnr) THEN
		RAISE EXCEPTION 'Auftragsposition ist nicht vorhanden';
	END IF;

	SELECT sonderrabatt INTO srabatt FROM AktKundendaten k JOIN Auftrag a ON k.knr = a.knr WHERE a.anr = fanr;
	SELECT a.rabatt INTO prabatt FROM APosition a WHERE a.anr = fanr AND a.apnr = fapnr;

	IF EXISTS (SELECT * FROM SonstigePosition p WHERE p.anr = fanr AND p.apnr = fapnr) THEN
		SELECT p.wert INTO vwert 
		FROM SonstigePosition p 
		WHERE p.anr = fanr AND p.apnr = fapnr;
		found := true;
	END IF;

	IF EXISTS (SELECT * FROM Leistungsposition p WHERE p.anr = fanr AND p.apnr = fapnr) THEN
		SELECT preis*menge INTO vwert 
		FROM Leistungsposition p JOIN LeistungsPreis l ON p.leistung = l.lnr 
		WHERE p.anr = fanr AND p.apnr = fapnr;
		found := true;
	END IF;

	IF (found) THEN
            vwert := vwert - vwert/100*prabatt;
	    vwert := vwert - vwert/100*srabatt;

	    RETURN vwert;
	ELSE
	    RAISE EXCEPTION 'Keine Sonstige oder Leistungsposition gefunden!';
	END IF;

	
	RETURN NULL;
END;
$$ LANGUAGE plpgsql;
